"""reCAPTCHA v3 solver for Playwright."""
from playwright_recaptcha.recaptchav3.async_solver import AsyncSolver
from playwright_recaptcha.recaptchav3.sync_solver import SyncSolver

__all__ = ["AsyncSolver", "SyncSolver"]



from __future__ import annotations

import asyncio
import functools
import io
import random
import re
from concurrent.futures import ThreadPoolExecutor
from typing import Any, Optional

import httpx
import pydub
import speech_recognition
from playwright.async_api import Page, Response

from playwright_recaptcha.errors import (
    RecaptchaNotFoundError,
    RecaptchaRateLimitError,
    RecaptchaSolveError,
)
from playwright_recaptcha.recaptchav2.recaptcha_box import AsyncRecaptchaBox


class AsyncSolver:
    """
    A class used to solve reCAPTCHA v2 asynchronously.

    Parameters
    ----------
    page : Page
        The playwright page to solve the reCAPTCHA on.
    attempts : int, optional
        The number of solve attempts, by default 3.

    Attributes
    ----------
    token : Optional[str]
        The g-recaptcha-response token.

    Methods
    -------
    close() -> None
        Remove the userverify response listener.
    solve_recaptcha(attempts: Optional[int] = None) -> str
        Solve the reCAPTCHA and return the g-recaptcha-response token.

    Raises
    ------
    RecaptchaNotFoundError
        If the reCAPTCHA was not found.
    RecaptchaRateLimitError
        If the reCAPTCHA rate limit has been exceeded.
    RecaptchaSolveError
        If the reCAPTCHA could not be solved.
    """

    def __init__(self, page: Page, attempts: int = 3) -> None:
        self._page = page
        self._attempts = attempts
        self.token: Optional[str] = None

    def __repr__(self) -> str:
        return f"AsyncSolver(page={self._page!r}, attempts={self._attempts!r})"

    async def __aenter__(self) -> AsyncSolver:
        return self

    async def __aexit__(self, *args: Any) -> None:
        self.close()

    @staticmethod
    async def _convert_audio_to_text(audio_url: str) -> Optional[str]:
        """
        Convert the reCAPTCHA audio to text.

        Parameters
        ----------
        audio_url : str
            The reCAPTCHA audio URL.

        Returns
        -------
        Optional[str]
            The reCAPTCHA audio text.
        """
        loop = asyncio.get_event_loop()

        async with httpx.AsyncClient() as client:
            response = await client.get(audio_url)

        wav_audio = io.BytesIO()
        mp3_audio = io.BytesIO(response.content)

        with ThreadPoolExecutor() as executor:
            audio = await loop.run_in_executor(
                executor, pydub.AudioSegment.from_mp3, mp3_audio
            )

            await loop.run_in_executor(
                executor, functools.partial(audio.export, wav_audio, format="wav")
            )

            recognizer = speech_recognition.Recognizer()

            with speech_recognition.AudioFile(wav_audio) as source:
                audio_data = await loop.run_in_executor(
                    executor, recognizer.record, source
                )

            text = await loop.run_in_executor(
                executor,
                functools.partial(
                    recognizer.recognize_google, audio_data, show_all=True
                ),
            )

        return text["alternative"][0]["transcript"] if text else None

    async def _random_delay(self) -> None:
        """Delay the execution for a random amount of time between 1 and 4 seconds."""
        await self._page.wait_for_timeout(random.randint(1000, 4000))

    async def _extract_token(self, response: Response) -> None:
        """
        Extract the g-recaptcha-response token from the userverify response.

        Parameters
        ----------
        response : Response
            The response to extract the g-recaptcha-response token from.
        """
        if re.search("/recaptcha/(api2|enterprise)/userverify", response.url) is None:
            return

        token_match = re.search('"uvresp","(.*?)"', await response.text())

        if token_match is not None:
            self.token = token_match.group(1)

    async def _get_audio_url(self, recaptcha_box: AsyncRecaptchaBox) -> str:
        """
        Get the reCAPTCHA audio URL.

        Parameters
        ----------
        recaptcha_box : AsyncRecaptchaBox
            The reCAPTCHA box.

        Returns
        -------
        str
            The reCAPTCHA audio URL.

        Raises
        ------
        RecaptchaRateLimitError
            If the reCAPTCHA rate limit has been exceeded.
        """
        if await recaptcha_box.audio_challenge_button.is_visible():
            await recaptcha_box.audio_challenge_button.click(force=True)

        while True:
            if await recaptcha_box.audio_challenge_is_visible():
                break

            if await recaptcha_box.rate_limit_is_visible():
                raise RecaptchaRateLimitError

            await self._page.wait_for_timeout(100)

        return await recaptcha_box.audio_download_button.get_attribute("href")

    async def _submit_audio_text(
        self, recaptcha_box: AsyncRecaptchaBox, text: str
    ) -> None:
        """
        Submit the reCAPTCHA audio text.

        Parameters
        ----------
        recaptcha_box : AsyncRecaptchaBox
            The reCAPTCHA box.
        text : str
            The reCAPTCHA audio text.

        Raises
        ------
        RecaptchaRateLimitError
            If the reCAPTCHA rate limit has been exceeded.
        """
        await recaptcha_box.audio_challenge_textbox.fill(text)
        await recaptcha_box.audio_challenge_verify_button.click()

        while recaptcha_box.frames_are_attached():
            if (
                await recaptcha_box.checkbox.is_checked()
                or await recaptcha_box.solve_failure_is_visible()
            ):
                break

            if await recaptcha_box.rate_limit_is_visible():
                raise RecaptchaRateLimitError

            await self._page.wait_for_timeout(100)

    def close(self) -> None:
        """Remove the userverify response listener."""
        try:
            self._page.remove_listener("response", self._extract_token)
        except KeyError:
            pass

    async def solve_recaptcha(self, attempts: Optional[int] = None) -> str:
        """
        Solve the reCAPTCHA and return the g-recaptcha-response token.

        Parameters
        ----------
        attempts : Optional[int], optional
            The number of solve attempts, by default 3.

        Returns
        -------
        str
            The g-recaptcha-response token.

        Raises
        ------
        RecaptchaNotFoundError
            If the reCAPTCHA was not found.
        RecaptchaRateLimitError
            If the reCAPTCHA rate limit has been exceeded.
        RecaptchaSolveError
            If the reCAPTCHA could not be solved.
        """
        self.token = None
        self._page.on("response", self._extract_token)

        attempts = attempts or self._attempts
        recaptcha_box = await AsyncRecaptchaBox.from_frames(self._page.frames)

        if await recaptcha_box.checkbox.is_hidden():
            raise RecaptchaNotFoundError

        await recaptcha_box.checkbox.click(force=True)

        while True:
            if (
                await recaptcha_box.audio_challenge_is_visible()
                or await recaptcha_box.audio_challenge_button.is_visible()
                and await recaptcha_box.audio_challenge_button.is_enabled()
            ):
                break

            if (
                not recaptcha_box.frames_are_attached()
                or await recaptcha_box.checkbox.is_checked()
            ):
                if self.token is None:
                    raise RecaptchaSolveError

                return self.token

            await self._page.wait_for_timeout(100)

        while attempts > 0:
            await self._random_delay()
            url = await self._get_audio_url(recaptcha_box)
            text = await self._convert_audio_to_text(url)

            if text is None:
                await recaptcha_box.new_challenge_button.click()
                attempts -= 1
                continue

            await self._random_delay()
            await self._submit_audio_text(recaptcha_box, text)

            if (
                not recaptcha_box.frames_are_attached()
                or await recaptcha_box.checkbox.is_checked()
            ):
                if self.token is None:
                    raise RecaptchaSolveError

                return self.token

            await recaptcha_box.new_challenge_button.click()
            attempts -= 1

        raise RecaptchaSolveError



from __future__ import annotations

import re
import time
from typing import Any, Optional

from playwright.async_api import Page, Response

from playwright_recaptcha.errors import RecaptchaTimeoutError, RecaptchaVersionError


class AsyncSolver:
    """
    A class used to solve reCAPTCHA v3 asynchronously.

    Parameters
    ----------
    page : Page
        The playwright page to solve the reCAPTCHA on.
    timeout : int, optional
        The solve timeout in seconds, by default 30.

    Attributes
    ----------
    token : Optional[str]
        The g-recaptcha-response token.

    Methods
    -------
    close() -> None
        Remove the reload response listener.
    solve_recaptcha(timeout: Optional[int] = None) -> str
        Solve the reCAPTCHA and return the g-recaptcha-response token.

    Raises
    ------
    RecaptchaTimeoutError
        If the solve timeout has been exceeded.
    RecaptchaVersionError
        If the reCAPTCHA is not version 3.
    """

    def __init__(self, page: Page, timeout: int = 30) -> None:
        self._page = page
        self._timeout = timeout
        self.token: Optional[str] = None

    def __repr__(self) -> str:
        return f"AsyncSolver(page={self._page!r}, timeout={self._timeout!r})"

    async def __aenter__(self) -> AsyncSolver:
        return self

    async def __aexit__(self, *args: Any) -> None:
        self.close()

    async def _extract_token(self, response: Response) -> None:
        """
        Extract the g-recaptcha-response token from the userverify response.

        Parameters
        ----------
        response : Response
            The response to extract the g-recaptcha-response token from.
        """
        if re.search("/recaptcha/(api2|enterprise)/reload", response.url) is None:
            return

        token_match = re.search('"rresp","(.*?)"', await response.text())

        if token_match is not None:
            self.token = token_match.group(1)

    def close(self) -> None:
        """Remove the reload response listener."""
        try:
            self._page.remove_listener("response", self._extract_token)
        except KeyError:
            pass

    async def solve_recaptcha(self, timeout: Optional[int] = None) -> str:
        """
        Solve the reCAPTCHA and return the g-recaptcha-response token.

        Parameters
        ----------
        timeout : Optional[int], optional
            The solve timeout in seconds, by default 30.

        Returns
        -------
        str
            The g-recaptcha-response token.

        Raises
        ------
        RecaptchaTimeoutError
            If the solve timeout has been exceeded.
        RecaptchaVersionError
            If the reCAPTCHA is not version 3.
        """
        self.token = None
        self._page.on("response", self._extract_token)

        timeout = timeout or self._timeout
        start_time = time.time()

        while self.token is None:
            if time.time() - start_time >= timeout:
                raise RecaptchaTimeoutError

            await self._page.wait_for_timeout(100)

        if len(self.token) > 600:
            raise RecaptchaVersionError

        return self.token



import pytest
from playwright.sync_api import sync_playwright

from playwright_recaptcha import (
    RecaptchaTimeoutError,
    RecaptchaVersionError,
    recaptchav3,
)


def test_solver() -> None:
    """Test the solver with a normal browser."""
    with sync_playwright() as playwright:
        browser = playwright.firefox.launch()
        page = browser.new_page()
        page.goto("https://antcpt.com/score_detector/")

        with recaptchav3.SyncSolver(page) as solver:
            solver.solve_recaptcha()


def test_solver_with_slow_browser() -> None:
    """Test the solver with a slow browser."""
    with sync_playwright() as playwright:
        browser = playwright.firefox.launch(slow_mo=1000)
        page = browser.new_page()
        page.goto("https://antcpt.com/score_detector/")

        with recaptchav3.SyncSolver(page) as solver:
            solver.solve_recaptcha()


def test_recaptcha_not_found() -> None:
    """Test the solver with a page that does not have a reCAPTCHA."""
    with sync_playwright() as playwright:
        browser = playwright.firefox.launch()
        page = browser.new_page()
        page.goto("https://www.google.com/")

        with pytest.raises(RecaptchaTimeoutError), recaptchav3.SyncSolver(
            page, timeout=10
        ) as solver:
            solver.solve_recaptcha()


def test_recaptcha_version_error() -> None:
    """Test the solver with a page that has a reCAPTCHA v2."""
    with sync_playwright() as playwright:
        browser = playwright.firefox.launch()
        page = browser.new_page()
        page.goto("https://cobra.ehr.com/ESS/Home/Login.aspx")

        with pytest.raises(RecaptchaVersionError), recaptchav3.SyncSolver(
            page
        ) as solver:
            solver.solve_recaptcha()



import pytest
from playwright.async_api import async_playwright

from playwright_recaptcha import (
    RecaptchaTimeoutError,
    RecaptchaVersionError,
    recaptchav3,
)


@pytest.mark.asyncio
async def test_solver() -> None:
    """Test the solver with a normal browser."""
    async with async_playwright() as playwright:
        browser = await playwright.firefox.launch()
        page = await browser.new_page()
        await page.goto("https://antcpt.com/score_detector/")

        async with recaptchav3.AsyncSolver(page) as solver:
            await solver.solve_recaptcha()


@pytest.mark.asyncio
async def test_solver_with_slow_browser() -> None:
    """Test the solver with a slow browser."""
    async with async_playwright() as playwright:
        browser = await playwright.firefox.launch(slow_mo=1000)
        page = await browser.new_page()
        await page.goto("https://antcpt.com/score_detector/")

        async with recaptchav3.AsyncSolver(page) as solver:
            await solver.solve_recaptcha()


@pytest.mark.asyncio
async def test_recaptcha_not_found() -> None:
    """Test the solver with a page that does not have a reCAPTCHA."""
    async with async_playwright() as playwright:
        browser = await playwright.firefox.launch()
        page = await browser.new_page()
        await page.goto("https://www.google.com/")

        with pytest.raises(RecaptchaTimeoutError):
            async with recaptchav3.AsyncSolver(page, timeout=10) as solver:
                await solver.solve_recaptcha()


@pytest.mark.asyncio
async def test_recaptcha_version_error() -> None:
    """Test the solver with a page that has a reCAPTCHA v2."""
    async with async_playwright() as playwright:
        browser = await playwright.firefox.launch()
        page = await browser.new_page()
        await page.goto("https://cobra.ehr.com/ESS/Home/Login.aspx")

        with pytest.raises(RecaptchaVersionError):
            async with recaptchav3.AsyncSolver(page) as solver:
                await solver.solve_recaptcha()



from typing import Optional


class RecaptchaError(Exception):
    """Base class for reCAPTCHA exceptions."""


class RecaptchaSolveError(RecaptchaError):
    """Base class for reCAPTCHA solve exceptions."""

    def __init__(self, message: Optional[str] = None) -> None:
        super().__init__(message or "The reCAPTCHA could not be solved.")


class RecaptchaNotFoundError(RecaptchaError):
    """An exception raised when the reCAPTCHA was not found."""

    def __init__(self) -> None:
        super().__init__("The reCAPTCHA was not found.")


class RecaptchaVersionError(RecaptchaSolveError):
    """An exception raised when the reCAPTCHA v3 solver is used for reCAPTCHA v2."""

    def __init__(self) -> None:
        super().__init__("The reCAPTCHA is not version 3.")


class RecaptchaRateLimitError(RecaptchaSolveError):
    """An exception raised when the reCAPTCHA rate limit has been exceeded."""

    def __init__(self) -> None:
        super().__init__("The reCAPTCHA rate limit has been exceeded.")


class RecaptchaTimeoutError(RecaptchaSolveError):
    """An exception raised when the reCAPTCHA solve timeout has been exceeded."""

    def __init__(self) -> None:
        super().__init__("The reCAPTCHA solve timeout has been exceeded.")



import pytest
from playwright.async_api import async_playwright

from playwright_recaptcha import (
    RecaptchaNotFoundError,
    RecaptchaRateLimitError,
    recaptchav2,
)


@pytest.mark.asyncio
@pytest.mark.xfail(raises=RecaptchaRateLimitError)
async def test_solver() -> None:
    """Test the solver with a normal browser."""
    async with async_playwright() as playwright:
        browser = await playwright.firefox.launch()
        page = await browser.new_page()

        await page.goto(
            "https://www.google.com/recaptcha/api2/demo", wait_until="networkidle"
        )

        async with recaptchav2.AsyncSolver(page) as solver:
            await solver.solve_recaptcha()


@pytest.mark.asyncio
@pytest.mark.xfail(raises=RecaptchaRateLimitError)
async def test_solver_with_slow_browser() -> None:
    """Test the solver with a slow browser."""
    async with async_playwright() as playwright:
        browser = await playwright.firefox.launch(slow_mo=1000)
        page = await browser.new_page()

        await page.goto(
            "https://www.google.com/recaptcha/api2/demo", wait_until="networkidle"
        )

        async with recaptchav2.AsyncSolver(page) as solver:
            await solver.solve_recaptcha()


@pytest.mark.asyncio
async def test_recaptcha_not_found() -> None:
    """Test the solver with a page that does not have a reCAPTCHA."""
    async with async_playwright() as playwright:
        browser = await playwright.firefox.launch()
        page = await browser.new_page()
        await page.goto("https://www.google.com/")

        with pytest.raises(RecaptchaNotFoundError):
            async with recaptchav2.AsyncSolver(page) as solver:
                await solver.solve_recaptcha()



import pytest
from playwright.sync_api import sync_playwright

from playwright_recaptcha import (
    RecaptchaNotFoundError,
    RecaptchaRateLimitError,
    recaptchav2,
)


@pytest.mark.xfail(raises=RecaptchaRateLimitError)
def test_solver() -> None:
    """Test the solver with a normal browser."""
    with sync_playwright() as playwright:
        browser = playwright.firefox.launch()
        page = browser.new_page()

        page.goto(
            "https://www.google.com/recaptcha/api2/demo", wait_until="networkidle"
        )

        with recaptchav2.SyncSolver(page) as solver:
            solver.solve_recaptcha()


@pytest.mark.xfail(raises=RecaptchaRateLimitError)
def test_solver_with_slow_browser() -> None:
    """Test the solver with a slow browser."""
    with sync_playwright() as playwright:
        browser = playwright.firefox.launch(slow_mo=1000)
        page = browser.new_page()

        page.goto(
            "https://www.google.com/recaptcha/api2/demo", wait_until="networkidle"
        )

        with recaptchav2.SyncSolver(page) as solver:
            solver.solve_recaptcha()


def test_recaptcha_not_found() -> None:
    """Test the solver with a page that does not have a reCAPTCHA."""
    with sync_playwright() as playwright:
        browser = playwright.firefox.launch()
        page = browser.new_page()
        page.goto("https://www.google.com/")

        with pytest.raises(RecaptchaNotFoundError), recaptchav2.SyncSolver(
            page
        ) as solver:
            solver.solve_recaptcha()



from __future__ import annotations

import re
from abc import ABC, abstractmethod
from typing import Iterable, List, Tuple, Union

from playwright.async_api import Frame as AsyncFrame
from playwright.async_api import Locator as AsyncLocator
from playwright.sync_api import Frame as SyncFrame
from playwright.sync_api import Locator as SyncLocator

from playwright_recaptcha.errors import RecaptchaNotFoundError, RecaptchaSolveError

Locator = Union[AsyncLocator, SyncLocator]
Frame = Union[AsyncFrame, SyncFrame]


class RecaptchaBox(ABC):
    """
    The base class for reCAPTCHA v2 boxes.

    Attributes
    ----------
    anchor_frame : Frame
        The reCAPTCHA anchor frame.
    bframe_frame : Frame
        The reCAPTCHA bframe frame.
    checkbox : Locator
        The reCAPTCHA checkbox locator.
    audio_challenge_button : Locator
        The reCAPTCHA audio challenge button locator.
    new_challenge_button : Locator
        The reCAPTCHA new challenge button locator.
    audio_download_button : Locator
        The reCAPTCHA audio download button locator.
    audio_challenge_textbox : Locator
        The reCAPTCHA audio challenge textbox locator.
    audio_challenge_verify_button : Locator
        The reCAPTCHA audio challenge verify button locator.

    Methods
    -------
    from_frames(frames: Iterable[Frame]) -> Union[AsyncRecaptchaBox, SyncRecaptchaBox]
        Create a reCAPTCHA box using a list of frames.
    frames_are_attached() -> bool
        Check if the reCAPTCHA frames are attached.
    rate_limit_is_visible() -> bool
        Check if the reCAPTCHA rate limit message is visible.
    solve_failure_is_visible() -> bool
        Check if the reCAPTCHA solve failure message is visible.
    audio_challenge_is_visible() -> bool
        Check if the reCAPTCHA audio challenge is visible.

    Raises
    ------
    RecaptchaNotFoundError
        If the reCAPTCHA was not found.
    RecaptchaSolveError
        If no unchecked reCAPTCHA boxes were found.
    """

    @staticmethod
    def _get_recaptcha_frame_pairs(
        frames: Iterable[Frame],
    ) -> List[Tuple[Frame, Frame]]:
        """
        Get the reCAPTCHA anchor and bframe frame pairs.

        Parameters
        ----------
        frames : Iterable[Frame]
            A list of frames to search for the reCAPTCHA anchor and bframe frames.

        Returns
        -------
        List[Tuple[Frame, Frame]]
            A list of reCAPTCHA anchor and bframe frame pairs.

        Raises
        ------
        RecaptchaNotFoundError
            If no reCAPTCHA anchor and bframe frame pairs were found.
        """
        anchor_frames = list(
            filter(
                lambda frame: re.search(
                    "/recaptcha/(api2|enterprise)/anchor", frame.url
                )
                is not None,
                frames,
            )
        )

        bframe_frames = list(
            filter(
                lambda frame: re.search(
                    "/recaptcha/(api2|enterprise)/bframe", frame.url
                )
                is not None,
                frames,
            )
        )

        frame_pairs = []

        for anchor_frame in anchor_frames:
            frame_id = anchor_frame.name[2:]

            for bframe_frame in bframe_frames:
                if frame_id in bframe_frame.name:
                    frame_pairs.append((anchor_frame, bframe_frame))

        if not frame_pairs:
            raise RecaptchaNotFoundError

        return frame_pairs

    @property
    def checkbox(self) -> Locator:
        """The reCAPTCHA checkbox locator."""
        return self.anchor_frame.get_by_role("checkbox", name="I'm not a robot")

    @property
    def audio_challenge_button(self) -> Locator:
        """The reCAPTCHA audio challenge button locator."""
        return self.bframe_frame.get_by_role("button", name="Get an audio challenge")

    @property
    def new_challenge_button(self) -> Locator:
        """The reCAPTCHA new challenge button locator."""
        return self.bframe_frame.get_by_role("button", name="Get a new challenge")

    @property
    def audio_download_button(self) -> Locator:
        """The reCAPTCHA audio download button locator."""
        return self.bframe_frame.get_by_role(
            "link", name="Alternatively, download audio as MP3"
        )

    @property
    def audio_challenge_textbox(self) -> Locator:
        """The reCAPTCHA audio challenge textbox locator."""
        return self.bframe_frame.get_by_role("textbox", name="Enter what you hear")

    @property
    def audio_challenge_verify_button(self) -> Locator:
        """The reCAPTCHA audio challenge verify button locator."""
        return self.bframe_frame.get_by_role("button", name="Verify")

    def frames_are_attached(self) -> bool:
        """
        Check if the reCAPTCHA frames are attached.

        Returns
        -------
        bool
            True if the reCAPTCHA frames are attached, False otherwise.
        """
        return (
            not self.bframe_frame.is_detached() and not self.anchor_frame.is_detached()
        )

    @property
    @abstractmethod
    def anchor_frame(self) -> Frame:
        """The reCAPTCHA anchor frame."""

    @property
    @abstractmethod
    def bframe_frame(self) -> Frame:
        """The reCAPTCHA bframe frame."""

    @classmethod
    @abstractmethod
    def from_frames(
        cls,
        frames: Iterable[Frame],
    ) -> Union[AsyncRecaptchaBox, SyncRecaptchaBox]:
        """
        Create a reCAPTCHA box using a list of frames.

        Parameters
        ----------
        frames : Iterable[Frame]
            A list of frames to search for the reCAPTCHA frames.

        Returns
        -------
        Union[AsyncRecaptchaBox, SyncRecaptchaBox]
            The reCAPTCHA box.

        Raises
        ------
        RecaptchaNotFoundError
            If the reCAPTCHA frames were not found.
        RecaptchaSolveError
            If no unchecked reCAPTCHA boxes were found.
        """

    @abstractmethod
    def rate_limit_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA rate limit message is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA rate limit message is visible, False otherwise.
        """

    @abstractmethod
    def solve_failure_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA solve failure message is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA solve failure message is visible, False otherwise.
        """

    @abstractmethod
    def audio_challenge_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA audio challenge is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA audio challenge is visible, False otherwise.
        """


class SyncRecaptchaBox(RecaptchaBox):
    """
    The synchronous class for reCAPTCHA v2 boxes.

    Parameters
    ----------
    anchor_frame : SyncFrame
        The reCAPTCHA anchor frame.
    bframe_frame : SyncFrame
        The reCAPTCHA bframe frame.

    Attributes
    ----------
    anchor_frame : Frame
        The reCAPTCHA anchor frame.
    bframe_frame : Frame
        The reCAPTCHA bframe frame.
    checkbox : Locator
        The reCAPTCHA checkbox locator.
    audio_challenge_button : Locator
        The reCAPTCHA audio challenge button locator.
    new_challenge_button : Locator
        The reCAPTCHA new challenge button locator.
    audio_download_button : Locator
        The reCAPTCHA audio download button locator.
    audio_challenge_textbox : Locator
        The reCAPTCHA audio challenge textbox locator.
    audio_challenge_verify_button : Locator
        The reCAPTCHA audio challenge verify button locator.

    Methods
    -------
    from_frames(frames: Iterable[SyncFrame]) -> SyncRecaptchaBox
        Create a reCAPTCHA box using a list of frames.
    frames_are_attached() -> bool
        Check if the reCAPTCHA frames are attached.
    rate_limit_is_visible() -> bool
        Check if the reCAPTCHA rate limit message is visible.
    solve_failure_is_visible() -> bool
        Check if the reCAPTCHA solve failure message is visible.
    audio_challenge_is_visible() -> bool
        Check if the reCAPTCHA audio challenge is visible.

    Raises
    ------
    RecaptchaNotFoundError
        If the reCAPTCHA was not found.
    RecaptchaSolveError
        If no unchecked reCAPTCHA boxes were found.
    """

    def __init__(self, anchor_frame: SyncFrame, bframe_frame: SyncFrame) -> None:
        self._anchor_frame = anchor_frame
        self._bframe_frame = bframe_frame

    def __repr__(self) -> str:
        return f"SyncRecaptchaBox(anchor_frame={self._anchor_frame!r}, bframe_frame={self._bframe_frame!r})"

    @classmethod
    def from_frames(cls, frames: Iterable[SyncFrame]) -> SyncRecaptchaBox:
        """
        Create a reCAPTCHA box using a list of frames.

        Parameters
        ----------
        frames : Iterable[SyncFrame]
            A list of frames to search for the reCAPTCHA frames.

        Returns
        -------
        SyncRecaptchaBox
            The reCAPTCHA box.

        Raises
        ------
        RecaptchaNotFoundError
            If the reCAPTCHA frames were not found.
        RecaptchaSolveError
            If no unchecked reCAPTCHA boxes were found.
        """
        frame_pairs = cls._get_recaptcha_frame_pairs(frames)

        for anchor_frame, bframe_frame in frame_pairs:
            if not anchor_frame.get_by_role(
                "checkbox", name="I'm not a robot"
            ).is_checked():
                return cls(anchor_frame, bframe_frame)

        raise RecaptchaSolveError("No unchecked reCAPTCHA boxes were found.")

    @property
    def anchor_frame(self) -> SyncFrame:
        """The reCAPTCHA anchor frame."""
        return self._anchor_frame

    @property
    def bframe_frame(self) -> SyncFrame:
        """The reCAPTCHA bframe frame."""
        return self._bframe_frame

    def rate_limit_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA rate limit message is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA rate limit message is visible, False otherwise.
        """
        return self.bframe_frame.get_by_text("Try again later").is_visible()

    def solve_failure_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA solve failure message is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA solve failure message is visible, False otherwise.
        """
        return self.bframe_frame.get_by_text(
            "Multiple correct solutions required - please solve more."
        ).is_visible()

    def audio_challenge_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA audio challenge is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA audio challenge is visible, False otherwise.
        """
        return self.bframe_frame.get_by_text("Press PLAY to listen").is_visible()


class AsyncRecaptchaBox(RecaptchaBox):
    """
    The asynchronous class for reCAPTCHA v2 boxes.

    Parameters
    ----------
    anchor_frame : AsyncFrame
        The reCAPTCHA anchor frame.
    bframe_frame : AsyncFrame
        The reCAPTCHA bframe frame.

    Attributes
    ----------
    anchor_frame : Frame
        The reCAPTCHA anchor frame.
    bframe_frame : Frame
        The reCAPTCHA bframe frame.
    checkbox : Locator
        The reCAPTCHA checkbox locator.
    audio_challenge_button : Locator
        The reCAPTCHA audio challenge button locator.
    new_challenge_button : Locator
        The reCAPTCHA new challenge button locator.
    audio_download_button : Locator
        The reCAPTCHA audio download button locator.
    audio_challenge_textbox : Locator
        The reCAPTCHA audio challenge textbox locator.
    audio_challenge_verify_button : Locator
        The reCAPTCHA audio challenge verify button locator.

    Methods
    -------
    from_frames(frames: Iterable[AsyncFrame]) -> AsyncRecaptchaBox
        Create a reCAPTCHA box using a list of frames.
    frames_are_attached() -> bool
        Check if the reCAPTCHA frames are attached.
    rate_limit_is_visible() -> bool
        Check if the reCAPTCHA rate limit message is visible.
    solve_failure_is_visible() -> bool
        Check if the reCAPTCHA solve failure message is visible.
    audio_challenge_is_visible() -> bool
        Check if the reCAPTCHA audio challenge is visible.

    Raises
    ------
    RecaptchaNotFoundError
        If the reCAPTCHA was not found.
    RecaptchaSolveError
        If no unchecked reCAPTCHA boxes were found.
    """

    def __init__(self, anchor_frame: AsyncFrame, bframe_frame: AsyncFrame) -> None:
        self._anchor_frame = anchor_frame
        self._bframe_frame = bframe_frame

    def __repr__(self) -> str:
        return f"AsyncRecaptchaBox(anchor_frame={self._anchor_frame!r}, bframe_frame={self._bframe_frame!r})"

    @classmethod
    async def from_frames(cls, frames: Iterable[AsyncFrame]) -> AsyncRecaptchaBox:
        """
        Create a reCAPTCHA box using a list of frames.

        Parameters
        ----------
        frames : Iterable[AsyncFrame]
            A list of frames to search for the reCAPTCHA frames.

        Returns
        -------
        AsyncRecaptchaBox
            The reCAPTCHA box.

        Raises
        ------
        RecaptchaNotFoundError
            If the reCAPTCHA frames were not found.
        RecaptchaSolveError
            If no unchecked reCAPTCHA boxes were found.
        """
        frame_pairs = cls._get_recaptcha_frame_pairs(frames)

        for anchor_frame, bframe_frame in frame_pairs:
            if not await anchor_frame.get_by_role(
                "checkbox", name="I'm not a robot"
            ).is_checked():
                return cls(anchor_frame, bframe_frame)

        raise RecaptchaSolveError("No unchecked reCAPTCHA boxes were found.")

    @property
    def anchor_frame(self) -> AsyncFrame:
        """The reCAPTCHA anchor frame."""
        return self._anchor_frame

    @property
    def bframe_frame(self) -> AsyncFrame:
        """The reCAPTCHA bframe frame."""
        return self._bframe_frame

    async def rate_limit_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA rate limit message is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA rate limit message is visible, False otherwise.
        """
        return await self.bframe_frame.get_by_text("Try again later").is_visible()

    async def solve_failure_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA solve failure message is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA solve failure message is visible, False otherwise.
        """
        return await self.bframe_frame.get_by_text(
            "Multiple correct solutions required - please solve more."
        ).is_visible()

    async def audio_challenge_is_visible(self) -> bool:
        """
        Check if the reCAPTCHA audio challenge is visible.

        Returns
        -------
        bool
            True if the reCAPTCHA audio challenge is visible, False otherwise.
        """
        return await self.bframe_frame.get_by_text("Press PLAY to listen").is_visible()



"""A library for solving reCAPTCHA v2 and v3 with Playwright."""
from playwright_recaptcha.errors import (
    RecaptchaError,
    RecaptchaNotFoundError,
    RecaptchaRateLimitError,
    RecaptchaSolveError,
    RecaptchaTimeoutError,
    RecaptchaVersionError,
)
from playwright_recaptcha.recaptchav2.async_solver import AsyncSolver as AsyncSolverV2
from playwright_recaptcha.recaptchav2.sync_solver import SyncSolver as SyncSolverV2
from playwright_recaptcha.recaptchav3.async_solver import AsyncSolver as AsyncSolverV3
from playwright_recaptcha.recaptchav3.sync_solver import SyncSolver as SyncSolverV3

__all__ = [
    "RecaptchaError",
    "RecaptchaNotFoundError",
    "RecaptchaRateLimitError",
    "RecaptchaSolveError",
    "RecaptchaTimeoutError",
    "RecaptchaVersionError",
    "AsyncSolverV2",
    "SyncSolverV2",
    "AsyncSolverV3",
    "SyncSolverV3",
]



"""reCAPTCHA v2 solver for Playwright."""
from playwright_recaptcha.recaptchav2.async_solver import AsyncSolver
from playwright_recaptcha.recaptchav2.sync_solver import SyncSolver

__all__ = ["AsyncSolver", "SyncSolver"]



from setuptools import find_packages, setup

with open("README.md", encoding="utf-8") as file:
    long_description = file.read()

setup(
    name="playwright-recaptcha",
    version="0.1.1",
    author="Xewdy444",
    author_email="xewdy@xewdy.tech",
    description="A library for solving reCAPTCHA v2 and v3 with Playwright",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Xewdy444/Playwright-reCAPTCHA",
    packages=find_packages(),
    python_requires=">=3.8",
    install_requires=[
        "httpx==0.23.3",
        "playwright==1.31.1",
        "pydub==0.25.1",
        "SpeechRecognition==3.9.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Topic :: Internet :: WWW/HTTP :: Browsers",
        "Framework :: AsyncIO",
    ],
)
